unit ephemerides;

interface

uses
  {SysUtils, Types, Classes, Variants, QTypes, QGraphics, QControls, QForms,
  QDialogs, QStdCtrls, IdComponent, IdTCPConnection, IdTCPClient, IdFTP,
  IdBaseComponent, IdAntiFreezeBase, IdAntiFreeze, QComCtrls; }
  ecliptic,temps,astromath,math, {soleil, } terre_VSOP;

type
   T_position = record
      alpha, delta, dist_T, dist_S,Magnitude : real;
   end;
   T_Orbite = record
      JJEpoque,
      DemiAxe,AnomMoy,
      ArgPer, LongNod,
      Inclin, Excent,H,G : real;
   end;
   T_Observatoire = record
      Longitude,Latitude,Altitude : real;
      RhoCosphiP,RhoSinPhiP       : real;
      Nom,code : string;
   end;

procedure calcul_ephemerides(JJephem_date : real; db : boolean; var orbite : T_Orbite; var position : T_position);
function StringHMS(angle:real;unites : string):string;
procedure Topocentrique(position : T_position; observatoire : T_Observatoire; TSL : real;
                        Var AlphaP,DeltaP : real);
procedure Lever(AlphaP,DeltaP : real; observatoire : T_Observatoire; var TSLL,TSLC,TSLM : double);
function Anomalie_Excentrique(db : boolean; var Astre : t_Orbite; anomalie_moyenne : real):real;

var
   obliquite_moyenne : real;

implementation

const
   Kc = 0.0057755;  { Constante de passage d'UA en jour-lumire }
   Ae = 6378140.0/1.49597870E11; { rayon terrestre exprim en UA }
   UA = 1.49597870E11;           { unit astronomique }
var
   t : textfile;


function Anomalie_Excentrique(db : boolean; var Astre : t_Orbite; anomalie_moyenne : real):real;
var
   U0,Un,W,ExcentR : real;
begin

   if db then writeln(t,'Rsolution de l''equation de Kepler.');

   with Astre do
   begin
      if Excent<0.99 then            { Resolution de l'Equation de Kepler }
      begin
         if db then Writeln(t,'Cas elliptique.');
         Un:=anomalie_moyenne;
         if (Excent>=0.95) and (anomalie_moyenne<>0)
            then Un:=sign(anomalie_moyenne)*Puiss(6*abs(anomalie_moyenne),1.0/3.0);
         ExcentR:=Excent; {/180*pi;}
         repeat
            U0:=Un;
            Un:=Un-(Un-ExcentR*sin(Un)-Anomalie_moyenne)/(1-ExcentR*cos(Un));
            if db then WRITELN(t,'UN                   = ',UN/PI*180:9:5,' x');
         until Abs(U0-Un)<1E-8
      end;
      if (Excent>=0.99) and (Excent<=1.01) then
      begin
         if db then Writeln(t,'Cas parabolique.');
         W:=3*Anomalie_moyenne/2;
         Un:=Puiss((W+sqrt(W*W+1)),1.0/3.0)-Puiss(sqrt(W*W+1)-W,1.0/3.0);
         if db then WRITELN(t,'W                    = ',W:9:5);
         if db then WRITELN(t,'UN                   = ',UN:9:5,' x');
      end;
      if Excent>1.01 then
      begin
         if db then Writeln(t,'Cas hyperbolique.');
         Un:=1;
         repeat
            U0:=Un;
            Un:=2*Un*(Excent-Un*(1-Anomalie_moyenne-ln(Un)))/(Excent*(Un*Un+1)-2*Un);
            if db then WRITELN(t,'UN                   = ',UN:9:5,' x');
         until Abs(U0-Un)<1E-8
      end
   end;
   Un:=Modulo(Un,2*Pi);
   if Un>Pi then Un:=Un-2*Pi;
   Anomalie_Excentrique:=Un;
end;

function StringHMS(angle:real;unites : string):string;
var
   hh,min,sec : real;
   s1,s2,s3 : string;
begin
    HMS(angle,hh,min,sec);
    Str(hh:3:0,S1);
    Str(min:2:0,S2);
    Str(sec:4:1,s3);
    StringHMS:=S1+Copy(unites,1,1)+s2+Copy(unites,2,1)+S3+copy(unites,3,1);
end;

procedure calcul_ephemerides(JJephem_date : real; db : boolean;
                             var orbite : T_Orbite;
                             var position : T_position);
{
   Calcul de l'phmrides (positions) en fonction du jour julien et de l'orbite donns
   db : si  vrai, on sort le detail dans un fichier log pour recupration par l'appelant
}
var
   AnoMoyDate : real;
   TC,L_sol,B_sol,R_sol,EpsZero,X_Sol,Y_Sol,Z_Sol,
   XS,YS,ZS,XX,XY,XZ,YX,YY,YZ,ZX,ZY,ZZ,
   GF,GP,GG,GQ,GH,GR,Phi1,Phi2,CosBeta,Beta : Double;
   {lSoleil,rSoleil, }  N,lappSoleil : real;
   hh,min,sec,Temps_Lumiere              : real;
   AnomalieExcentrique                   : real;
   RayonVecteur,AnomalieVraie,Un         : real;
   GA,PA,GB,PB,GC,PC,Px,Py,Pz,GX,GY,GZ   : real;
   boucle                                : integer;

begin
   AssignFile(t,'ephem.log');
   rewrite(t);

{   PSoleil(JJEphem_date,lSoleil,rSoleil,N,lappSoleil);
  ou
    Soleil_Precis(db,JJEphem_date,lSoleil,rSoleil,N,lappSoleil);
}
   TC:=(JJEphem_Date-2451545.0 )/365250;
   L_sol:=Terre_L(TC)+Pi; if L_sol>2*Pi then L_sol:=L_sol-2*Pi;
   B_sol:=-Terre_B(TC); if B_sol<-Pi then B_sol:=B_sol+2*Pi;
   R_sol:=Terre_R(TC);


   if db then
   begin
      HMS(L_Sol*180/pi,hh,min,sec);
      Writeln(t,'Longitude vraie du Soleil : ',HH:3:0,'',min:2:0,'''',sec:5:2,'"  = ',L_Sol*180/pi:12:8,'');
      HMS(B_Sol*180/pi,hh,min,sec);
      Writeln(t,'Latitude vraie du Soleil  : ',HH:3:0,'',min:2:0,'''',sec:5:2,'"  = ',B_Sol*180/pi:12:8,'');
      Writeln(t,'Distance Terre-Soleil     :   ',R_Sol:10:8,' UA');
   end;
   EpsilonJJNutation(JJEphem_date,Obliquite_moyenne);
   HMS(obliquite_moyenne*180/pi,hh,min,sec);
   if db then
   Writeln(t,'Obliquit moyenne              : ',HH:3:0,'',min:2:0,'''',sec:5:2,'"  = ',obliquite_moyenne*180/pi:12:8,'');

   EpsZero:=Eps0(TC*10);
   Ecliptic2Rect(L_sol,B_sol,R_sol,EpsZero,XS,YS,ZS);
{
   XS:=RSoleil*Cos(LSoleil);
   YS:=RSoleil*Sin(LSoleil)*cos(Obliquite_moyenne);
   ZS:=RSoleil*Sin(LSoleil)*sin(Obliquite_moyenne);   }
{ Passage  l'quinoxe 2000 }
   CoefPrecessionDateVers2000(JJEphem_date, XX,XY,XZ,YX,YY,YZ,ZX,ZY,ZZ);

   X_Sol:=Xx * Xs + YX * Ys + Zx * Zs;
   Y_sol:=Xy * Xs + Yy * Ys + Zy * Zs;
   Z_sol:=Xz * Xs + Yz * Ys + Zz * Zs;


   if db then
   begin
      Writeln(t,'Xs  (J2000.0)                  :   ',X_Sol:10:8,' U.A.');
      Writeln(t,'Ys     "                       :   ',Y_Sol:10:8,' U.A.');
      Writeln(t,'Zs     "                       :   ',Z_Sol:10:8,' U.A.');
   end;
   with orbite do
   begin
      GF:=cos(LongNod);
      GP:=-sin(LongNod)*cos(Inclin);
      GG:=cos(Obliquite_moyenne)*sin(LongNod);
      GQ:=cos(Inclin)*cos(LongNod)*cos(Obliquite_moyenne)-sin(Inclin)*sin(Obliquite_moyenne);
      GH:=sin(Obliquite_moyenne)*sin(LongNod);
      GR:=cos(Longnod)*cos(Inclin)*sin(Obliquite_moyenne)+sin(Inclin)*cos(Obliquite_moyenne);
   end;
   Polaire(GP,GF,PA,GA);
   Polaire(GQ,GG,PB,GB);
   Polaire(GR,GH,PC,GC);
   if db then
   begin
      Writeln(t,'F : ',GF:12:8,'  P : ',GP:12:8);
      Writeln(t,'G : ',GG:12:8,'  Q : ',GQ:12:8);
      Writeln(t,'H : ',GH:12:8,'  R : ',GR:12:8);
      Writeln(t,'A : ',GA/pi*180:12:8,'','  a : ',PA:12:8);
      Writeln(t,'B : ',GB/pi*180:12:8,'','  b : ',PB:12:8);
      Writeln(t,'C : ',GC/pi*180:12:8,'','  c : ',PC:12:8);
   end;

   Temps_Lumiere:=0;
   for boucle:=1 to 2 do
   begin
      with orbite do
         AnomoyDate:=AnomMoy+0.01720209895/DemiAxe/sqrt(DemiAxe)*(JJephem_date-Temps_Lumiere-JJEpoque);
      AnomoyDate:=Modulo(AnoMoydate,2*pi);
      {if AnomoyDate<0 then AnoMoyDate:=AnoMoyDate+2*Pi;}
      if  AnomoyDate>Pi then AnoMoyDate:=AnoMoyDate-2*Pi;

      HMS(AnoMoyDate*180/pi,hh,min,sec);
      if db then
         Writeln(t,'Anomalie Moyenne               : ',HH:3:0,'',min:2:0,'''',sec:5:2,'"  = ',AnoMoyDate*180/pi:10:6);


      AnomalieExcentrique:=Anomalie_Excentrique(db,orbite,AnoMoydate);
      HMS(AnomalieExcentrique*180/pi,hh,min,sec);
      if db then
         Writeln(t,'Anomalie Excentrique           : ',HH:3:0,'',min:2:0,'''',sec:5:2,'"  = ',AnomalieExcentrique*180/pi:12:8,'');
      if db then
         WRITELN(t,'Exentricite                    : ',orbite.Excent:9:5);

      with orbite do
      if Excent < 0.99 then
     {Ellipse}
      begin
         RayonVecteur:=DemiAxe*(1-Excent*Cos(AnomalieExcentrique));
         AnomalieVraie:=2*arctan(tan(AnomalieExcentrique/2)*sqrt((1+Excent)/(1-Excent)));
      end
        { Polaire(DemiAxe*(1-Excent*Cos(AnomalieExcentrique)),
                 Demiaxe*sqrt(1-Excent*Excent)*sin(AnomalieExcentrique),
                 RayonVecteur,AnomalieVraie)
        }
      else
         if Excent >1 then {Hyperbole}
         begin end
         else begin        {Parabole}
         end;
      HMS(AnomalieVraie*180/pi,hh,min,sec);
      if db then
         Writeln(t,'Anomalie Vraie                 : ',HH:3:0,'',min:2:0,'''',sec:5:2,'"  = ',AnomalieVraie*180/pi:12:8,'');
      if db then
         Writeln(t,'Rayon Vecteur                  : ',rayonVecteur:9:5,' UA');

      with orbite do
      begin
         Px:=RayonVecteur*PA*sin(GA+ArgPer+AnomalieVraie);
         Py:=RayonVecteur*PB*sin(GB+ArgPer+AnomalieVraie);
         Pz:=RayonVecteur*PC*sin(GC+ArgPer+AnomalieVraie);
      end;

      GX:=Px+X_Sol;
      GY:=Py+Y_Sol;
      GZ:=Pz+Z_Sol;

      if db then
      begin
         Writeln(t,'Coord. Rect. quat. Hliocent. :   ');
         Writeln(t,'x                              :   ',Px:10:8,' UA');
         Writeln(t,'y                              :   ',Py:10:8,' UA');
         Writeln(t,'z                              :   ',Pz:10:8,' UA');
         Writeln(t,'Coord. Rect. quat. Gocent.   :   ');
         Writeln(t,'X                              :   ',Gx:10:8,' UA');
         Writeln(t,'Y                              :   ',Gy:10:8,' UA');
         Writeln(t,'Z                              :   ',Gz:10:8,' UA');
         Writeln(t);
      end;
      with Position do
      begin
         dist_T:=sqrt(GX*GX+GY*GY+GZ*GZ);
         Temps_Lumiere:=Kc*dist_T;
         dist_S:=RayonVecteur;
         Delta:=Arcsin(GZ/dist_T);
         Polaire(GX/dist_T/cos(Delta),GY/dist_T/cos(Delta),Un,Alpha);

         CosBeta:=(GX*Px+GY*Py+Gz*Pz)/Dist_T/RayonVecteur;
         Beta:=ArcCos(CosBeta);
         If (beta<(2/3*Pi)) and (Orbite.G<>0) and (Orbite.H<>0) then
         begin
            Phi1:=exp(-3.33*Puiss(tan(Beta/2),0.63));
            Phi2:=exp(-1.87*Puiss(tan(Beta/2),1.22));
            Magnitude:=Orbite.H+5*log10(RayonVecteur*Dist_T)-2.5*Log10((1-Orbite.G)*Phi1+Orbite.G*Phi2)
         end
         else Magnitude:=99;
         If alpha<0 then alpha:=alpha+2*pi;
         if abs(Un-1)>1E-7 then writeln(t,'Erreur de calcul de Alpha et Delta. (',Un:12:10,'<>1)');
         if db then
         begin
            HMS(Temps_lumiere*24,hh,min,sec);
            Writeln(t,'Temps de lumire               : ',HH:3:0,'h',min:2:0,'m',sec:5:2,'s');
            HMS(Alpha*12/pi,hh,min,sec);
            Writeln(t,'Ascension Droite               : ',HH:3:0,'h',min:2:0,'m',sec:5:2,'s  = ',Alpha*12/pi:12:8,'h');
            HMS(Delta*180/pi,hh,min,sec);
            Writeln(t,'Dclinaison                    : ',HH:3:0,'',min:2:0,'''',sec:5:2,'"  = ',Delta*180/pi:12:8,'');
            Writeln(t,'Distance  la Terre            :   ',Dist_T:10:8,' UA');
            Writeln(t);
         end;
      end;
   end; { Fin de la boucle d'itration sur le temps lumire }

   {      alpha, delta, dist_T, dist_S : real;}


   CloseFile(t);
end;

procedure Topocentrique(position : T_position; observatoire : T_Observatoire; TSL : real;
                        Var AlphaP,DeltaP : real);
{
  Renvoie les coordonnes apparentes alpha' et delta' pour objet donn par
  sa position gocentrique et sa distance (en UA),
  un observaoire donne par sa longitude (en degr) et ses donnes de
  latitude sur le goide terrestre (rho.cos(phi') et rho.sin(phi')
  le temps sidral local en radians
}
var
   AH,AHP,wb,qcosdeltap,q : real;
begin
   AH:=TSL-Position.alpha;
   wb:=AE/Position.dist_T;
   with position,observatoire do
   begin
      Polaire(cos(delta)*cos(AH)-rhocosphip*sin(wb),cos(delta)*sin(AH),qcosdeltap,AHP);
      Polaire(qcosdeltap,sin(delta)-rhosinphip*sin(wb),q,deltaP);
   end;
   AlphaP:=TSL-AHP;
   AlphaP:=Modulo(AlphaP,2*Pi);
   if AlphaP<0 then AlphaP:=AlphaP+2*Pi;
end;

procedure Lever(AlphaP,DeltaP : real; observatoire : T_Observatoire; var TSLL,TSLC,TSLM : double);
var
   dummy,H0: real;
begin
   H0:=Arccos(-tan(observatoire.Latitude)*tan(deltap));
   TSLL:=AlphaP+H0;
   TSLC:=AlphaP-H0;
   if TSLC>TSLL then begin dummy:=TSLC; TSLC:=TSLL; TSLL:=dummy; end;
   TSLL:=Modulo(TSLL,2*pi);
   TSLC:=Modulo(TSLC,2*pi);
end;

begin
end.